//
//  AppDelegate.swift
//  PowerGuard
//
//  Created by Phidgets on 2018-02-23.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = PowerGuard()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var settingsBox: NSBox!
    @IBOutlet weak var overVoltageSlider: NSSlider!
    @IBOutlet weak var minOverVoltageLabel: NSTextField!
    @IBOutlet weak var maxOverVoltageLabel: NSTextField!
    @IBOutlet weak var overVoltageLabel: NSTextField!
    @IBOutlet weak var fanModePopup: NSPopUpButton!
    @IBOutlet weak var powerEnabledCheck: NSButton!
    
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0
    
    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        do{
            let attachedDevice = sender as! PowerGuard
            let minOverVoltage = try attachedDevice.getMinOverVoltage()
            let maxOverVoltage = try attachedDevice.getMaxOverVoltage()
            let fanMode = try attachedDevice.getFanMode()
            let powerEnabled = try attachedDevice.getPowerEnabled()
            
            DispatchQueue.main.async {
                self.phidgetInfoBox.fillPhidgetInfo(device: sender)
                self.settingsBox.isHidden = false
                self.overVoltageSlider.minValue = minOverVoltage
                self.overVoltageSlider.maxValue = maxOverVoltage
                self.overVoltageSlider.doubleValue = minOverVoltage
                self.minOverVoltageLabel.stringValue = String(format:"%.2f", minOverVoltage)
                self.maxOverVoltageLabel.stringValue = String(format:"%.2f", maxOverVoltage)
                self.minOverVoltageLabel.stringValue = String(format:"%.2f V", minOverVoltage)
                self.powerEnabledCheck.state = powerEnabled ? NSControl.StateValue.on : NSControl.StateValue.off
                self.fanModePopup.addItem(withTitle: "On")
                self.fanModePopup.addItem(withTitle: "Off")
                self.fanModePopup.addItem(withTitle: "Auto")
                self.fanModePopup.selectItem(at: fanMode == FanMode.on ? 0 : fanMode == FanMode.off ? 1 : 2)
                
                //Adjusting window height
                var frame = self.window.frame
                frame.size.height = self.window.maxSize.height
                self.window.setFrame(frame, display: true, animate: false)
                self.window.center()
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.settingsBox.isHidden = true
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    
    //MARK: GUI Controls
    @IBAction func setOverVoltage(_ sender: Any) {
        do{
            let slider = sender as! NSSlider
            try ch.setOverVoltage(slider.doubleValue)
            overVoltageLabel.stringValue = String(format:"%.2f V",slider.doubleValue)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setFanMode(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            if(popup.indexOfSelectedItem == 0){
                try ch.setFanMode(FanMode.on)
            }
            else if(popup.indexOfSelectedItem == 0){
                try ch.setFanMode(FanMode.off)
            }
            else{
                try ch.setFanMode(FanMode.auto)
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setPowerEnabled(_ sender: Any) {
        do{
            let check = sender as! NSButton
            try ch.setPowerEnabled(check.state == NSControl.StateValue.on ? true : false)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
}

